using UnityEditor;
//Created by Neodrop
//neodrop@unity3d.ru
using UnityEngine;

[CustomEditor(typeof(CsBiped))]
public class CsBipedEditor : Editor 
{
    public Texture bodyTexture;
    CsBiped targetObject;
    SerializedObject serializedObject;
    static int w = 339, h = 341;

	void OnEnable () 
    {
        targetObject = target as CsBiped;
        serializedObject = new SerializedObject(target);
        serializedObject.Update();
        if (bodyTexture == null)
        {
            bodyTexture = AssetDatabase.LoadAssetAtPath(AntaresGameObjectCustom.Routines.pathToImages + "gold.jpg", typeof(Texture)) as Texture;
            string bipName = "";
            foreach (Transform tr in targetObject.gameObject.GetComponentsInChildren<Transform>())
            {
                bipName = tr.gameObject.name;
                switch(bipName)
                {
                    case "Bip01 R Hand" : 
                        targetObject.handRight = tr;
                        break;
                    case "Bip01 L Hand" : 
                        targetObject.handLeft = tr;
                        break;
                    case "Bip01 Head" : 
                        targetObject.head = tr;
                        break;
                    case "Bip01 R Toe0" : 
                        targetObject.footRight = tr;
                        break;
                    case "Bip01 L Toe0" : 
                        targetObject.footLeft = tr;
                        break;
                }
            }
        }
        serializedObject.ApplyModifiedProperties();
	}
	
	public override void OnInspectorGUI () 
    {
        if (!bodyTexture) return; 
        Rect lRect = GUILayoutUtility.GetLastRect();
        AntaresGameObjectCustom.Routines.DrawHeader("ANTARES BIPED BODY");
        float middle = lRect.width / 2, half = 170f;
        Vector2 leftUp = new Vector2(middle - half, lRect.y + 64);
        GUI.Box(new Rect(leftUp.x, leftUp.y, w, h), bodyTexture);

        EditorGUILayout.BeginHorizontal();
        float yHand = leftUp.y + 107;
        if (GUI.Button(new Rect(leftUp.x + 45, yHand, 15, 15), new GUIContent(":", "Right Hand"))) EditorGUIUtility.PingObject(targetObject.handRight.gameObject);
        if(GUI.Button(new Rect(leftUp.x + 279, yHand, 15, 15), new GUIContent(":", "Left Hand"))) EditorGUIUtility.PingObject(targetObject.handLeft.gameObject);
        float xFoot = (lRect.y + h - 10) + 36;
        if (GUI.Button(new Rect(middle - 80, xFoot, 15, 15), new GUIContent(".", "Right Foot"))) EditorGUIUtility.PingObject(targetObject.footRight.gameObject);
        if (GUI.Button(new Rect(middle + 74, xFoot, 15, 15), new GUIContent(".", "Left Foot"))) EditorGUIUtility.PingObject(targetObject.footLeft.gameObject);
        EditorGUILayout.EndHorizontal();

        if(GUI.Button(new Rect(middle - 7.5f, lRect.y + 121, 15, 15), new GUIContent("+", "Head"))) EditorGUIUtility.PingObject(targetObject.head.gameObject);
        GUILayout.Space(h + 20);
	    AntaresGameObjectCustom.Routines.DrawFooter();
    }

    [MenuItem(("Antares/Biped/Add Biped finder module"))]
    public static void AddBipedFinder()
    {
        Selection.activeGameObject.AddComponent<CsBiped>();
    }

}
